@@ -126,7 +126,7 @@ class _BcachefsFileBinary(io.BufferedIOBase):
     """
 
     def __init__(self, name, extents, file, inode, size):
-        self.name = file
+        self.name = name
         self._inode = inode
         self._size = size
 
@@ -221,11 +221,16 @@ class _BcachefsFileBinary(io.BufferedIOBase):
         if self._extent_pos >= len(self._extents):
             return 0
 
+        if not isinstance(b, memoryview):
+            b = memoryview(b)
+
         # continue reading the current extent
         extent = self._extents[self._extent_pos]
 
         self._file.seek(extent.offset + self._extent_read)
-        read = self._file.readinto(b)
+        read = self._file.readinto(
+            b[: min(len(b), extent.size - self._extent_read)]
+        )
 
         self._extent_read += read
         self._pos += read
@@ -238,6 +243,7 @@ class _BcachefsFileBinary(io.BufferedIOBase):
         # finished reading the file
         if self._pos > self._size:
             diff = self._pos - self._size
+            self._pos = self._size
 
             self._extent_pos += 1
             self._extent_read = 0
